/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.management;

import java.io.IOException;
import java.util.Properties;

import org.ow2.petals.tools.webconsole.services.AbstractServiceFactory;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.utils.ConnexionHelper;

/**
 * @author aruffie
 * Factory for recover the
 * management service implementation
 */
public class ManagementServiceFactory extends AbstractServiceFactory {

    private static ManagementServiceFactory instance = null;
    private ManagementService service = null;
    private String host = null;
    private String port = null;
    private String login = null;
    private String password = null;
    /**
     * function can allow to recover the
     * factory instance
     */
    public static synchronized AbstractServiceFactory getInstance() {
        if(instance == null){
            instance = new ManagementServiceFactory();
        }
        return instance;
    }

    /**
     * Method allow to recover the management
     * service from another layer
     */
    @Override
    public synchronized ManagementService getService(String hostname, Integer jmxJNDIPort,
                                                     String jmxLogin, String jmxPassword)
                                                     throws PetalsServiceException {
        if(this.service == null || (!hostname.equals(this.host))
           || (!jmxJNDIPort.equals(this.port)) || (!jmxLogin.equals(this.login))
           || (!jmxPassword.equals(this.password))){
            this.service = this.getImplementation(hostname, jmxJNDIPort, jmxLogin,
                                                  jmxPassword);
        }
        return this.service;
    }

    /**
     * Method allow to create the management
     * service implementation
     */
    private synchronized ManagementService getImplementation(String hostname,
                                                             Integer jmxJNDIPort,
                                                             String jmxLogin,
                                                             String jmxPassword)
                                                             throws PetalsServiceException{
        
        final Properties props = new Properties();
        
        // recover classloader
            
            // load services.properties file thank to classloader
            try {
                props.load(this.getClass().getClassLoader().getResourceAsStream("services.properties"));
            } catch (IOException e) {
                throw new PetalsServiceException(e);
            }
            final String className = props.getProperty("management.service.impl");
            Class loadedClass = null;
            try {
                    loadedClass = Class.forName(className);
            } catch (ClassNotFoundException e) {
                throw new PetalsServiceException(e);
            }
            boolean checkManagementServiceImplementation = Boolean.FALSE;
            
            Class[] interfaces;
            interfaces = loadedClass.getInterfaces();
            for(final Class inter : interfaces){
                if(inter.equals(ManagementService.class)){
                    checkManagementServiceImplementation = Boolean.TRUE;
                }
            }
            if(checkManagementServiceImplementation){     
                ManagementService managementService = null;
                try {
                    managementService = (ManagementService)
                        loadedClass.newInstance();
                } catch (InstantiationException e) {
                    throw new PetalsServiceException(e);
                } catch (IllegalAccessException e) {
                    throw new PetalsServiceException(e);
                }
                managementService.setJmxClient(ConnexionHelper
                        .connect(hostname, jmxJNDIPort, jmxLogin, jmxPassword));
                return managementService;
                       
            } else {
                throw new PetalsServiceException(loadedClass.getName()+" don't " +
                        "implement "+ManagementService.class);
            }
       

    }
}
