/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.management;

import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanAttributeInfo;
import javax.management.ObjectName;

import org.ow2.petals.jmx.ComponentClient;
import org.ow2.petals.jmx.ConfigurationInstallerComponentClient;
import org.ow2.petals.jmx.DeploymentServiceClient;
import org.ow2.petals.jmx.InstallationServiceClient;
import org.ow2.petals.jmx.InstallerComponentClient;
import org.ow2.petals.jmx.JMXClient;
import org.ow2.petals.jmx.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.ComponentErrorException;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.jmx.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceErrorException;
import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationErrorException;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.w3c.dom.Document;

/**
 * @author aruffie this class provide a management service implementation, thank
 *         to JMX technology
 */
public class ManagementServiceJMXImpl implements ManagementService {

    private JMXClient jmxClient;
    private String loadedInstaller;
    public void startComponent(final String componentName) throws PetalsServiceException {
        try {

            // check component existence

            final ComponentClient cc = this.jmxClient
                    .getComponentClient(componentName);
            if (ComponentClient.State.STARTED.equals(cc.getState())) {
                throw new PetalsServiceException("Component: "+componentName+" already started");
            }

            // Start the component if this state is shutdown or stopped
            if ((ComponentClient.State.SHUTDOWN.equals(cc.getState()))
                    || (ComponentClient.State.STOPPED.equals(cc.getState()))) {
                cc.start();
            } else {
                throw new PetalsServiceException(
                        "Can't perform start action, because the component: " + componentName
                                + " is in the following state: " + cc.getState());
            }

            /*
             * Wrap all exception type in PetalsServiceException, to pick up the
             * same exception type
             */
        } catch (ComponentDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (ComponentErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void stopComponent(final String componentName) throws PetalsServiceException {
        try {

            // check component existence
            final ComponentClient cc = this.jmxClient
                    .getComponentClient(componentName);

            if (ComponentClient.State.STARTED.equals(cc.getState())) {
                cc.stop();
            } else {
                throw new PetalsServiceException("Can't stop the component: " + componentName
                        + " because, is it in the following state: " + cc.getState());
            }

            /*
             * Wrap all exception type in PetalsServiceException, to pick up the
             * same exception type
             */
        } catch (ComponentDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (ComponentErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }

    }

    public void shutdownComponent(final String componentName) throws PetalsServiceException {
        try {

            // check component existence
            final ComponentClient cc = this.jmxClient
                    .getComponentClient(componentName);

            if (ComponentClient.State.STOPPED.equals(cc.getState())) {
                cc.shutdown();
            } else {
                throw new PetalsServiceException("Can't shutdown the component: " + componentName
                        + " because, is it in the following state: " + cc.getState());
            }
        } catch (ComponentDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (ComponentErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }

    }

    public ObjectName[] componentsInventory() throws PetalsServiceException {
        try {
            return this.jmxClient.getAdminServiceClient().getBindingComponents();
        } catch (final AdminServiceErrorException e) {
            throw new PetalsServiceException("Can't retrieve components from petals", e);
        } catch (final AdminDoesNotExistException e) {
            throw new PetalsServiceException("Can't find PetalsAdmin service", e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public ObjectName[] enginesInventory() throws PetalsServiceException {
        try {
            return this.jmxClient.getAdminServiceClient().getEngineComponents();
        } catch (final AdminServiceErrorException e) {
            throw new PetalsServiceException("Can't retrieve components from petals", e);
        } catch (final AdminDoesNotExistException e) {
            throw new PetalsServiceException("Can't find PetalsAdmin service", e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public String[] serviceAssembliesInventory() throws PetalsServiceException {
        try {
            return this.jmxClient.getDeploymentServiceClient().getDeployedServiceAssemblies();
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Deployment service", e);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public String[] sharedLibrariesInventory() throws PetalsServiceException {
        try {
            return this.jmxClient.getInstallationServiceClient()
                    .getInstalledSharedLibraries();
        } catch (InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void installComponent(final URL fileURL, final Map<MBeanAttributeInfo, Object> attributes, final String loadedInstaller) throws PetalsServiceException{
        try {
            final InstallerComponentClient icc = this.jmxClient
                    .getInstallationServiceClient().loadInstaller(loadedInstaller);
            if(!attributes.isEmpty()){
                final ConfigurationInstallerComponentClient ccic = icc.getConfigurationInstallerClient();
                try {
                    ccic.setAttributes(attributes);
                } catch (AttributeErrorException e) {
                    throw new PetalsServiceException(e);
                }
            }
            icc.install();
            
            /*
             * MBeanInfo mbeanInfo = this.petalsJMXClient.getMBeanServerConnection().getMBeanInfo(picc.getInstallerConfigurationMBean());
             *                mbeanInfo.getAttributes()
             */
        } catch (final PerformActionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Installation service", e);
        } catch (final InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void installServiceAssembly(final URL fileURL) throws PetalsServiceException {
        try {
            this.jmxClient.getDeploymentServiceClient().deploy(fileURL);
        } catch (DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void installSharedLibrary(final URL sharedLibraryURL) throws PetalsServiceException {
        try {
            this.jmxClient.getInstallationServiceClient().installSharedLibrary(
                    sharedLibraryURL);
        } catch (InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void startServiceAssembly(final String saName) throws PetalsServiceException {
        try {
            // check if the SA is already deployed
            final DeploymentServiceClient dsc = this.jmxClient
                    .getDeploymentServiceClient();
            if (dsc.isServiceAssemblyDeployed(saName)) {
                // heck its state to know if we can start if
                if (DeploymentServiceClient.State.SHUTDOWN.equals(dsc.getState(saName))
                        || DeploymentServiceClient.State.STOPPED
                                .equals(dsc.getState(saName))) {
                    dsc.start(saName);
                } else {
                    if (DeploymentServiceClient.State.STARTED.equals(dsc.getState(saName))) {
                        // SA already started
                        throw new PetalsServiceException(saName
                                + " service-assembly already started");
                    } else {
                        throw new PetalsServiceException("Can't start: " + saName
                                + " service assembly, " + "because is in the following state: "
                                + dsc.getState(saName));
                    }
                }
            } else {
                throw new PetalsServiceException(saName + " service-assembly is not deployed.");
            }
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Deployment service", e);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void stopServiceAssembly(final String saName) throws PetalsServiceException {
        try {
            // check if the SA is already deployed
            final DeploymentServiceClient dsc = this.jmxClient
                    .getDeploymentServiceClient();
            if (this.jmxClient.getDeploymentServiceClient().isServiceAssemblyDeployed(saName)) {
                // check its state to know if we can stop it
                if (DeploymentServiceClient.State.STARTED.equals(dsc.getState(saName))) {
                    dsc.stop(saName);
                } else {
                    throw new PetalsServiceException("Can't stop " + saName + " service assembly, "
                            + "because is in the following state: " + dsc.getState(saName));
                }
            } else {
                throw new PetalsServiceException("Can't perform stop action, because the service-"
                        + "assembly: " + saName + " is not deployed.");
            }
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Deployment service", e);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }

    }

    public void uninstallComponent(final String componentName) throws PetalsServiceException {
        try {
           
            final InstallerComponentClient icc = this.jmxClient.
                getInstallationServiceClient().loadInstaller(componentName);
            if(icc.isInstalled()){
                icc.uninstall();
            }
            this.jmxClient.getInstallationServiceClient().unloadInstaller(componentName);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final PerformActionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void uninstallServiceAssembly(final String saName) throws PetalsServiceException {
        try {
            this.jmxClient.getDeploymentServiceClient().undeploy(saName);
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Deployment service", e);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void uninstallSharedLibrary(final String sharedLibraryName)
            throws PetalsServiceException {
        try {
            if(!this.jmxClient.getInstallationServiceClient().uninstallSharedLibrary(
                    sharedLibraryName)){
                throw new PetalsServiceException("Unable to uninstall '"+sharedLibraryName+"' shared library, check petals terminal");
            }
        } catch (InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void shutdownServiceAssembly(String saName) throws PetalsServiceException {

        try {
            // check if SA already deployed
            final DeploymentServiceClient dsc = this.jmxClient
                    .getDeploymentServiceClient();

            if (dsc.isServiceAssemblyDeployed(saName)) {

                // check its state to know of we can shutdown it
                if (DeploymentServiceClient.State.STOPPED.equals(dsc.getState(saName))) {
                    dsc.shutdown(saName);
                } else {
                    throw new PetalsServiceException("Can't shutdown the service assembly: "
                            + saName + " because, is in the following state: "
                            + dsc.getState(saName));
                }
            } else {
                throw new PetalsServiceException("This service assembly: " + saName
                        + ", is not deployed");
            }
        } catch (DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public JMXClient getJmxClient() throws PetalsServiceException {
        if (this.jmxClient != null) {
            return this.jmxClient;
        } else {
            throw new PetalsServiceException("PEtALS connection lost: PetalsJMXClient "
                    + this.jmxClient);
        }
    }

    public Set<Map<String, String>> getTopology() throws PetalsServiceException {
        try {
            final Set<Map<String, String>> topology = this.getJmxClient()
                    .getPetalsAdminServiceClient().retrieveTopology();
            return topology;
        } catch (final PetalsAdminServiceErrorException e) {
            throw new PetalsServiceException("Can't get petals topology", e);
        } catch (final PetalsAdminDoesNotExistException e) {
            throw new PetalsServiceException("Can't get petals topology", e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException("Can't get petals topology", e);
        }
    }

    public void setJmxClient(JMXClient jmxClient) throws PetalsServiceException {
        this.jmxClient = jmxClient;
    }

    public String getComponentState(String componentName) throws PetalsServiceException {
        ComponentClient cc = null;
        try {
            cc = this.getJmxClient().getComponentClient(componentName);
            return cc.getState();
        } catch (ComponentDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (ComponentErrorException e) {
            throw new PetalsServiceException(e);
        } catch (ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public List<Map<String, Object>> getAllEndpoints() throws PetalsServiceException {
        try {

            final List<Map<String, Object>> result = this.getJmxClient()
                    .getEndpointRegistryClient().getAllEndpoints();
            return result;

        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final EndpointRegistryServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Monitoring Service", e);
        } catch (final EndpointRegistryServiceErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public String getServiceAssemblyState(String sa) throws PetalsServiceException {
        try {
            return this.getJmxClient().getDeploymentServiceClient().getState(sa);
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Deployment service", e);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    /**
     * Allow to return all attributes owned
     * by a component at the url provided
     * @param fileURL
     * @return
     * @throws PetalsServiceException
     */
    public Map<MBeanAttributeInfo, Object> getConfigComponent(final URL fileURL) throws PetalsServiceException {
        try {
            final InstallerComponentClient icc = this.jmxClient
                    .getInstallationServiceClient().loadNewInstaller(fileURL);
            //picc.install();
            this.loadedInstaller = icc.getMBeanName().getKeyProperty("name");
            ConfigurationInstallerComponentClient cicc;
            Map<MBeanAttributeInfo, Object> attributes;
            try {
                    cicc = icc.getConfigurationInstallerClient();
                attributes = cicc.getAttributes();
            } catch (final AttributeErrorException e) {
                throw new PetalsServiceException(e);
            } catch (final PerformActionErrorException e) {
                throw new PetalsServiceException(e);
            } catch (final javax.management.RuntimeOperationsException e){
                attributes = new HashMap<MBeanAttributeInfo, Object>();
            }
            
            return attributes;
            
            /*
             * MBeanInfo mbeanInfo = this.petalsJMXClient.getMBeanServerConnection().getMBeanInfo(picc.getInstallerConfigurationMBean());
             *                mbeanInfo.getAttributes()
             */
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException("Unable to find Installation service", e);
        } catch (final InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }
    
    public Document getDocument(final String serviceName, final String endpointName)
                                   throws PetalsServiceException {
        Document result = null;
        if(
           serviceName != null &&
           !serviceName.equals("") && 
           endpointName != null &&
           !endpointName.equals("")
          ){
            try {
                result = this.getJmxClient()
                        .getEndpointRegistryClient().getDescription(serviceName, endpointName);
            } catch (final ConnectionErrorException e) {
                throw new PetalsServiceException(e);
            } catch (final EndpointRegistryServiceDoesNotExistException e) {
                throw new PetalsServiceException("Unable to find Monitoring Service", e);
            } catch (final EndpointRegistryServiceErrorException e) {
                throw new PetalsServiceException(e);
            }
        }
        return result;
    }

    public String getLoadedInstaller() {
        return loadedInstaller;
    }
    
    public Map<MBeanAttributeInfo, Object> recoverRuntimeAttributes(final String componentName) throws PetalsServiceException{
        
        Map<MBeanAttributeInfo, Object> attributes = null;
        RuntimeConfigurationComponentClient rcc = null;
        try {
            rcc = this.getJmxClient().getRuntimeConfigurationClient(componentName);
            attributes = rcc.getConfigurationMBeanAttributes();
        } catch (final RuntimeConfigurationDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (final RuntimeConfigurationErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (PerformActionErrorException e) {
            throw new PetalsServiceException(e);
        }
        return attributes;
    }
    
    public void setRuntimeAttributes(final Map<MBeanAttributeInfo, Object> attributes,final String componentName) throws PetalsServiceException{
        RuntimeConfigurationComponentClient rcc = null;
        try {
            rcc = this.getJmxClient().getRuntimeConfigurationClient(componentName);
            rcc.setAttributes(attributes);
        } catch (final RuntimeConfigurationDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (final RuntimeConfigurationErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (PerformActionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }

    public void forceUnloadInstaller(final String componentName) throws PetalsServiceException{
        InstallationServiceClient ics = null;
        try {
            ics = this.jmxClient.getInstallationServiceClient();
            ics.forceUnloadInstaller(componentName);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final  InstallationServiceDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (final  InstallationServiceErrorException e) {
            throw new PetalsServiceException(e);
        }
    }
    
    public void forceUnDeploy(final String serviceAssemblyName) throws PetalsServiceException{
    DeploymentServiceClient dsc = null;
        try {
            dsc = this.jmxClient.getDeploymentServiceClient();
            dsc.forceUndeploy(serviceAssemblyName);
        } catch (final DeploymentServiceErrorException e) {
            throw new PetalsServiceException(e);
        } catch (final DeploymentServiceDoesNotExistException e) {
            throw new PetalsServiceException(e);
        } catch (final ConnectionErrorException e) {
            throw new PetalsServiceException(e);
        }
    }
}
