/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.proxyfication;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.WSDLWriter;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.utils.SuGenerator;
import org.ow2.petals.tools.webconsole.util.FilePermissionHelper;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.StringHelper;

import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SaBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBeanForSa;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.XmlElement;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiXmlGenerator;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiZipper;

/**
 * 
 * @author aruffie
 * Proxyfication Service Implementation
 */
public class ProxyficationServiceImpl implements ProxyficationService {
    
    /**
     * Allow to generate a Service Assembly
     * which contain the proxy
     * @param soap
     * @param rest
     * @param serviceName
     * @param soapAddr
     * @param endpointNameProvider
     * @param wsdlVersion
     * @param interfaceNameProvider
     * @param interfaceNameNsUrlProvider
     * @param timeout
     * @param addrSUProvide
     * @param namespaceProvider
     * @param consumeURI
     * @param restConsumeURI
     * @param restNSPrefix
     * @param restNSURI
     * @param restRMPrefixResponse
     * @param wsdlFile
     * @param proxyName
     * @param repository
     * @param restConsumeSUName
     * @param soapConsumeSUName
     * @param wsdlProvideFile
     * @return
     * @throws PetalsServiceException
     */
    public File addProxy(boolean soap,boolean rest, final String serviceName,
                         final String soapAddr, final String endpointNameProvider,
                         final String wsdlVersion, final String interfaceNameProvider,
                         final String interfaceNameNsUrlProvider, final String timeout,
                         final String addrSUProvide, final String namespaceProvider,
                         final String soapConsumeURI, final String restConsumeURI,
                         final String restNSPrefix, final String restNSURI,
                         final String restRMPrefixResponse, final File wsdlFile,
                         final String proxyName, final File repository,
                         final String restConsumeSUName,final String soapConsumeSUName,
                         File wsdlProvideFile) throws PetalsServiceException{
        
        List<SuBean> suBeans = new ArrayList<SuBean>();
        
        if (!repository.exists()) {
            repository.mkdir();
        }
        try {
            FilePermissionHelper.setRights(repository, new int[] { 7, 7, 7 }, false);
        } catch (IOException e) {
            throw new PetalsServiceException(e);
        }
        
        
        // Add these SU in the su list of the SA
        if(!StringHelper.isNullOrEmpty(addrSUProvide)){
            suBeans.add(this.createSOAPProvideSU(serviceName,soapAddr,
                                                 endpointNameProvider,wsdlVersion,
                                                 interfaceNameProvider,
                                                 interfaceNameNsUrlProvider,
                                                 timeout,addrSUProvide,namespaceProvider,
                                                 wsdlFile,repository,wsdlProvideFile));
        }
        if(soap){
            final SuBean suSoapConsume = this.createSOAPConsumeSU(serviceName,
                                                                  soapConsumeURI,
                                                                  interfaceNameProvider,
                                                                  interfaceNameNsUrlProvider,
                                                                  addrSUProvide,timeout);
            suBeans.add(suSoapConsume);
        }
        //If rest
        if(rest){
            final SuBean suRestConsume = this.createRestConsumeSU(serviceName,
                                                                  restConsumeURI,
                                                                  restNSPrefix,
                                                                  restNSURI,
                                                                  restRMPrefixResponse,
                                                                  interfaceNameProvider,
                                                                  interfaceNameNsUrlProvider,
                                                                  soapConsumeURI,timeout,
                                                                  addrSUProvide);
            suBeans.add(suRestConsume);
        }
        
        // Generate a jbi.xml file for a service assembly.
        SaBean saBean = new SaBean();
        saBean.setSaName(proxyName);
        saBean.setDescription("SA generated for the " + proxyName);
        List<SuBeanForSa> sus = new ArrayList<SuBeanForSa>();
        List<File> suZipFiles = new ArrayList<File>();
        File saZipFile = null;
        for (SuBean suBean : suBeans) {

            // Generate a jbi.xml file a service unit.
            String jbiXmlForSu = JbiXmlGenerator.getInstance().generateJbiXmlFileForSu(suBean);

            File jbiSu = null;
            try {
                jbiSu = new File(repository.getCanonicalPath() + "/jbi.xml");
            } catch (IOException e) {
                throw new PetalsServiceException(e);
            }
           

            File suFile = null;
            
            try {
                FileWriter writer = new FileWriter(jbiSu);
                writer.write(jbiXmlForSu);
                writer.close();
                List<File> rootFiles = new ArrayList<File>();

                JbiZipper jbiZipper = JbiZipper.getInstance();
                // Save SU in the repository
 
                try {

                    rootFiles.add(jbiSu);
                    for(final XmlElement specificElement : suBean.specificElements){
                        if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                            suFile = new File(repository.getCanonicalPath() + "/" + restConsumeSUName
                                    + ".zip");
                        } else {
                            if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(soapConsumeURI))){
                                suFile = new File(repository.getCanonicalPath() + "/" + soapConsumeSUName
                                        + ".zip");
                            } else {
                                if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(SOAPMODE))){
                                    rootFiles.add(wsdlProvideFile);
                                    suFile = new File(repository.getCanonicalPath() + "/" + addrSUProvide
                                            + ".zip");
                                }
                            }
                        }
                    }
                    //écriture du fichier sur le disque
                    suZipFiles.add(jbiZipper.createSuZipFile(suFile, rootFiles));

                } catch (FileNotFoundException e) {    
                    throw new PetalsServiceException("FilenotFoundException in suFile initialization",e);
                }

            } catch (IOException e) {
                throw new PetalsServiceException("IOException occured when " + jbiSu.getName() + " was wrote",e);
            }

            SuBeanForSa suBeanForSa = new SuBeanForSa(suBean);
            suBeanForSa.setZipArtifact(suFile.getName());
            
            for(final XmlElement specificElement : suBean.specificElements){
                if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                    suBeanForSa.setSuName(restConsumeSUName);
                } else {
                    if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(soapConsumeURI))){
                        suBeanForSa.setSuName(soapConsumeSUName);
                    } else {
                        if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(SOAPMODE))){
                            suBeanForSa.setSuName(addrSUProvide);
                        }
                    }
                }
            }
            suBeanForSa.setComponentName(COMPONENTNAME);

            sus.add(suBeanForSa);

            jbiSu.delete();
        }
        saBean.setSus(sus);
        // Generate jbi file for the SA
        String jbiXmlForSa = JbiXmlGenerator.getInstance().generateJbiXmlFileForSa(saBean);
        File jbiSa;
        try {
            jbiSa = new File(repository.getCanonicalPath() + "/jbi.xml");
        } catch (IOException e) {
            throw new PetalsServiceException(e);
        }
        // Création des zip de la SA
        try {
            FileWriter writer = new FileWriter(jbiSa);
            writer.write(jbiXmlForSa);
            writer.close();
            JbiZipper jbiZipper = JbiZipper.getInstance();

            // Save the SA zip file in the repository
            File saFile = new File(repository.getCanonicalPath() + "/" + proxyName + ".zip");
            saZipFile = jbiZipper.createSaZipFile(saFile, suZipFiles, jbiSa);
        } catch (IOException e) {
            throw new PetalsServiceException("IOException occured when " + jbiSa.getName() + " was wrote",e);
        }
        jbiSa.delete();
        return saZipFile;
    }
    
    /**
     * Allow to create a REST Consume SU
     * for the specified addr Provider SU
     * and several other parameters
     * @param serviceName
     * @param restConsumeURI
     * @param restNSPrefix
     * @param restNSURI
     * @param restRMPrefixResponse
     * @param interfaceNameProvider
     * @param interfaceNameNsUrlProvider
     * @param consumeURI
     * @param timeout
     * @param addrSUProvide
     * @return
     */
    private SuBean createRestConsumeSU(final String serviceName,
                                       final String restConsumeURI,
                                       final String restNSPrefix,
                                       final String restNSURI,
                                       final String restRMPrefixResponse,
                                       final String interfaceNameProvider,
                                       final String interfaceNameNsUrlProvider,
                                       final String consumeURI,
                                       final String timeout,
                                       final String addrSUProvide){
        
        SuBean suBean = null;
        
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();

        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
   
        final String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(restConsumeURI);
        componentElementsConsumer.add(element);
        
        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue(RESTMODE);
        componentElementsConsumer.add(element);
        
        if(restNSPrefix != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-prefix");
            element.setValue(restNSPrefix);
            componentElementsConsumer.add(element);
        }
        if(restNSURI != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-uri");
            element.setValue(restNSURI);
            componentElementsConsumer.add(element);
        }
        if(restRMPrefixResponse != null){
            element = new XmlElement();
            element.setName("soap:rest-remove-prefix-on-response");
            element.setValue(restRMPrefixResponse);
            componentElementsConsumer.add(element);
        }
        // Create the Consumer SU
        suBean = SuGenerator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suBean;
    }

    /**
     * Allow to create a SOAP Consume SU
     * for the specified addr Provider SU
     * and several other parameters
     * @param serviceName
     * @param consumeURI
     * @param interfaceNameProvider
     * @param interfaceNameNsUrlProvider
     * @param addrSUProvide
     * @param timeout
     * @return
     */
    private SuBean createSOAPConsumeSU(final String serviceName,
                                       final String consumeURI,
                                       final String interfaceNameProvider,
                                       final String interfaceNameNsUrlProvider,
                                       final String addrSUProvide,
                                       final String timeout){
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();
        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
   
        final String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(consumeURI);
        componentElementsConsumer.add(element);

        // Create the Consumer SU
        SuBean suConsume = SuGenerator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suConsume;
    }

    /**
     * Allow to create a SOAP Provide SU
     * and several other parameters
     * @param serviceName
     * @param soapAddr
     * @param endpointNameProvider
     * @param wsdlVersion
     * @param interfaceNameProvider
     * @param interfaceNameNsUrlProvider
     * @param timeout
     * @param addrSUProvide
     * @param namespaceProvider
     * @param wsdlFile
     * @param repository
     * @param wsdlProvideFile
     * @return
     * @throws PetalsServiceException
     */
    private SuBean createSOAPProvideSU(final String serviceName,
                                       final String soapAddr,
                                       final String endpointNameProvider,
                                       final String wsdlVersion,
                                       final String interfaceNameProvider,
                                       final String interfaceNameNsUrlProvider,
                                       final String timeout,
                                       final String addrSUProvide,
                                       final String namespaceProvider,
                                       final File wsdlFile,
                                       final File repository,
                                       File wsdlProvideFile) throws PetalsServiceException{
        
        ArrayList<XmlElement> componentElementsProvider = new ArrayList<XmlElement>();
        
        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
        String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        /*
         * Begin Provide SOAP parameter
         */
        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(soapAddr);
        /* test de résolution d'address */

        /* fin résolution d'address*/
        componentElementsProvider.add(element);



        element = new XmlElement();
        element.setName("soap:soap-version");
        element.setValue(wsdlVersion);
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:add-root");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:chunked-mode");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:cleanup-transport");
        element.setValue("true");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue(SOAPMODE);
        componentElementsProvider.add(element);
        /*
         * Ending Provide SOAP parameter
         */
        
        try {
           wsdlProvideFile = modifySoapAddressWSDL(wsdlFile,repository,wsdlProvideFile,addrSUProvide);
        } catch (final IOException e1) {
            System.out.println("Error occured durring the creation of the new wsdl"
                    + " for the provide: " + e1.getMessage());
        } catch (WSDLException e) {
            throw new PetalsServiceException(e);
        } catch (URISyntaxException e) {
            throw new PetalsServiceException(e);
        }

        // Create the Consumer/Provider SU
        
        SuBean suProvide = SuGenerator.createSuBeanProvide(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", interfaceNameProvider, interfaceNameProvider, interfaceNameProvider,
                namespaceProvider, "operation", timeout, componentElementsProvider,wsdlProvideFile
                        .getName());
        return suProvide;
    }
    
    /**
     * Return the new wsdlFile with the new
     * address soap for the provide SU
     * @param wsdlFile
     * @param repository
     * @param addrSUProvide
     * @return
     * @throws IOException
     * @throws WSDLException
     * @throws URISyntaxException
     */
    public File modifySoapAddressWSDL(final File wsdlFile,
                                       final File repository,
                                       File wsdlProvide,
                                       final String addrSUProvide) throws IOException, WSDLException, URISyntaxException {
        //final String fileName = wsdlFile.getName().substring(0, wsdlFile.getName().indexOf("."));
        /*final File wsdlProvide = new File(repository.getCanonicalPath() + "/" + fileName
                + "Provide.wsdl");*/
        if (!wsdlProvide.exists()) {
            wsdlProvide.createNewFile();
        }
        FilePermissionHelper.setRights(wsdlProvide, new int[] { 7, 7, 7 }, false);

        // parse with easy wsdl
        final WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        final Description description = wsdlReader.read(wsdlFile.toURI().toURL());
        for (final Service service : description.getServices()) {
            for (final Endpoint endpoint : service.getEndpoints()) {
                endpoint.setAddress(addrSUProvide);
            }
        }
        final WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();

        GeneralHelper.writeXmlFile(wsdlWriter.getDocument(description), wsdlProvide);
        
        return wsdlProvide;
        
    }
}
