/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.rmiinjector.model;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;

import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.StringHelper;

/**
 * @author aruffie
 * 
 */
public class Response{

    private Fault fault;

    private ExchangeStatus status;

    private Exception error;

    private NormalizedMessage outMessage;

    private String timestamp;
    
    private Date date;

    private String endpoint;

    private String service;

    private String _interface;
    
    private String formatedContent;

    private List<Property> mEXProperties;
    
    public Fault getFault() {
        return fault;
    }

    public void setFault(Fault fault) {
        this.fault = fault;
    }

    public ExchangeStatus getStatus() {
        return status;
    }

    public String getStringStatus() {
        if (status.equals(ExchangeStatus.ACTIVE)) {
            return "ACTIVE";
        } else if (status.equals(ExchangeStatus.DONE)) {
            return "DONE";
        } else if (status.equals(ExchangeStatus.ERROR)) {
            return "ERROR";
        } else {
            return "ERROR";
        }

    }

    public void setStatus(ExchangeStatus status) {
        this.status = status;
    }

    public Exception getError() {
        return error;
    }

    public void setError(Exception error) {
        this.error = error;
    }

    public NormalizedMessage getOutMessage() {
        return outMessage;
    }

    public void setOutMessage(NormalizedMessage outMessage) {
        this.outMessage = outMessage;
    }

    /**
     * @param error
     * @param fault
     * @param outMessage
     * @param status
     */
    public Response(Exception error, Fault fault, NormalizedMessage outMessage,
            ExchangeStatus status) {
        super();
        this.error = error;
        this.fault = fault;
        this.outMessage = outMessage;
        this.status = status;
        this.timestamp = new java.rmi.dgc.VMID().toString() + " --> "
                + new GregorianCalendar().getTime();
        this.mEXProperties = new ArrayList<Property>();
    }

    public Response(final Exception error) {
        this.error = error;
    }

    public String getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(String timestamp) {
        this.timestamp = timestamp;
        this.timestamp = new java.rmi.dgc.VMID().toString() + " --> "
                + new GregorianCalendar().getTime();
    }

    public String getContent() {
        if (this.error != null) {
            return this.error.getMessage();
        } else if (this.fault != null) {
            return this.fault.getContent().toString();
        } else if (this.outMessage != null) {
            return this.outMessage.getContent().toString();
        } else {
            return "";
        }
    }

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public String getService() {
        return service;
    }

    public void setService(String service) {
        this.service = service;
    }

    public String get_interface() {
        return _interface;
    }

    public void set_interface(String _interface) {
        this._interface = _interface;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public String getFormatedContent() throws TransformerFactoryConfigurationError, IOException, TransformerException {
        if(formatedContent == null){
            String format = "no content for this message exchange";
            
            if(this.outMessage != null){
                format = StringHelper.createString(this.outMessage.getContent());
            }
            
            if(this.fault != null){
                format = StringHelper.createString(this.fault.getContent());
            }
            
            if(this.error != null){
                format = this.error.getMessage()+": \n "+this.error.getStackTrace();
            }
            
            formatedContent = format;
            format = null;
        }
        return StringHelper.prettyPrint(formatedContent);
    }

    public void setFormatedContent(String formatedContent) {
        this.formatedContent = formatedContent;
    }
    
    public String getLocalInterface(){
        final QName qname = QName.valueOf(this._interface);
        return qname.getLocalPart();
    }
    
    public String getLocalService(){
        final QName qname = QName.valueOf(this.service);
        return qname.getLocalPart();
    }
    
    public String getNameSpace(){
        final QName qname = QName.valueOf(this._interface);
        return qname.getNamespaceURI();
    }

    public List<Property> getMEXProperties() {
        return mEXProperties;
    }

    public void setMEXProperties(List<Property> properties) {
        mEXProperties = properties;
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        final Set attachments = this.outMessage.getAttachmentNames();
            final File slTree = new File(new URI(GeneralHelper.repoPath+"/attachements"));
            if(slTree != null && slTree.exists()){
                final List<File> attachFiles = Arrays.asList(slTree.listFiles());
                 final Iterator it = attachments.iterator();
                 while(it.hasNext()){
                     final String attachName = (String) it.next();
                     for(final File file : attachFiles){
                         if(file.getName().equals(attachName)){
                             file.delete();
                         }
                     }
                 }
             }
    }
    
    
}
