/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.petals.tools.webconsole.business.AddWSDLBBean;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.WSDLEXCEPTION_ERROR;
/**
 * @author aruffie
 * 
 */
public class AddWSDLUIBean {
    
    private boolean internalService;

    private AddWSDLBBean businessBean;
    
    private File wsdlProvided;

    private String wsdlVersion;

    private List<Service> services;

    private Service selectedService;

    private String selectedServiceName;
    
    public AddWSDLUIBean(){
        this.businessBean = new AddWSDLBBean();
    }
    
    /**
     * Allow to upload the provided wsdl
     * @param IFileUploadPart file
     * @param HttpServletRequest req
     * @throws NonLocalizedError
     */
    public void onUpload(final IFileUploadPart file, final HttpServletRequest req)
            throws NonLocalizedError{
        if (file.isEmptyStream()) {
            throw new IllegalArgumentException("WSDL file provided is empty");
        } else {
            try {
                this.wsdlProvided = this.businessBean.storeWSDL(file, req);
            } catch (IOException e) {
                throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
            } catch (URISyntaxException e) {
                final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            }
        }
    }

    /**
     * submit the form and if a service is seleted, return success to show the
     * ProxifyService page
     * @throws NonLocalizedError
     * @return String
     * 
     */
    public String submitAddWSDL() throws NonLocalizedError {
        Description description = null;
        try {
            description = GeneralHelper.readWSDL(this.wsdlProvided);
        } catch (WSDLException e) {
            throw new NonLocalizedError(WSDLEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (MalformedURLException e) {
            final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (IOException e) {
            final NonLocalizedError error = new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        this.services = description.getServices();
        this.wsdlVersion = description.getVersion().toString();
        return "success";
    }

    /**
     * Reset all informations like the checkbox and the wsdl provided file if it
     * exists into the filesystem.
     */
    public void resetAddWSDL() {
        this.internalService = false;
        if (this.wsdlProvided.exists()) {
            this.wsdlProvided.delete();
        }
        this.wsdlProvided = null;
    }

    public File getWsdlProvided() {
        return wsdlProvided;
    }

    public void setWsdlProvided(File wsdlProvided) {
        this.wsdlProvided = wsdlProvided;
    }

    public List<Service> getServices() {
        return services;
    }

    public void setServices(List<Service> services) {
        this.services = services;
    }

    public Service getSelectedService() {
        return selectedService;
    }

    public void setSelectedService(Service selectedService) {
        this.selectedService = selectedService;
    }

    public void resetSelectedService() {
        this.selectedService = null;
    }

    public String getWsdlVersion() {
        return wsdlVersion;
    }

    public void setWsdlVersion(String wsdlVersion) {
        this.wsdlVersion = wsdlVersion;
    }

    public String submitSelectedService() {
        
        /*
         * Find the selected service
         */
        this.selectedService = findService(this.selectedServiceName);
        if (this.selectedService != null) {
            return "success";
        } else {
            throw new IllegalArgumentException("nothing selected service");
        }
    }

    public boolean isInternalService() {
        return internalService;
    }

    public void setInternalService(boolean internalService) {
        this.internalService = internalService;
    }
    
    private Service findService(final String serviceName){
        for(final Service service : this.services){
            if(service.getQName().getLocalPart().equals(serviceName)){
                return service;
            }
        }
        return null;
    }

    public String getSelectedServiceName() {
        return selectedServiceName;
    }

    public void setSelectedServiceName(String selectedServiceName) {
        this.selectedServiceName = selectedServiceName;
    }
    
    
}
