/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanAttributeInfo;
import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.petals.jmx.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.tools.webconsole.business.ManagementBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.to.DomainTO;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.util.ConfigHelper;
import org.ow2.petals.tools.webconsole.util.TopologyHelper;
import org.xml.sax.SAXException;

/**
 * @author aruffie
 * 
 */
public class ManagementUIBean {

    private ManagementService managementService = null;

    private ManagementBBean businessBean = null;
    
    private DomainTO domain = null;

    private String host = null;

    private String port = null;

    private String login = null;

    private String password = null;
    
    private boolean failedConnection = true;

    private String currentServerName = null;
    
    private String componentName = null;
    
    public static final String NUMBER_FORMAT_ERROR = "Number format error";
    public static final String UNKNOW_HOST_ERROR = "Unknow host error";
    public static final String PETALS_SERVICE_ERROR = "Petals service error";
    public static final String PETALS_CONSOLE_ERROR = "Petals console error";
    public static final String CONFIGURATION_ERROR = "Configuration error";
    public static final String SAX_ERROR = "Sax exception error";
    public static final String PARSER_CONFIGURATION_ERROR = "Parser configuration error";
    public static final String FACTORY_CONFIGURATION_ERROR = "Factory configuration error";
    public static final String WSDLEXCEPTION_ERROR = "WSDL exception error";
    public static final String MALFORMED_URL_ERROR = "Malformed url error";
    public static final String IOEXCEPTION_ERROR = "IO exception error";
    public static final String URI_SYNTAX_ERROR = "Uri syntax error";
    public static final String FILE_NOT_FOUND_ERROR = "File note found error";
    public static final String RMI_SERVER_CONFIGURATION_ERROR = "Rmi server configuration error";
    public static final String ILLEGAL_ARGUMENT_ERROR = "Illegal argument error";
    public static final String CLASS_LOADER_ERROR = "Class loader error";
    public static final String CONFIGURATION_FILE_ERROR = "Configuration file error";
    public static final String SELECTED_ENDPOINT_ERROR = "Selected endpoint error";
    public static final String TRANSFORMER_ERROR = "Transformer error";
    public static final String TRANSFORMER_FACTORY_CONFIGURATION_ERROR = "Transformer factory configuration error";
    public static final String MESSAGING_ERROR = "Messaging error";
    public static final String DOWNLOAD_ATTACHMENT_ERROR = "Download attachment error";
    
    public ManagementUIBean(){
        this.businessBean = new ManagementBBean();
        try {
            connect();
        } catch (final NonLocalizedError e) {
            // nothing: lets the interface inform the user
            if(this.domain == null){
                this.domain = new DomainTO();
            }
        }
    }

    public void reconnect(final HttpServletRequest request) throws NonLocalizedError{
        serverConnection(request);
        ServerUIBean currentServerUIBean = (ServerUIBean) request.getSession().getAttribute("serverUIBean");
        if(currentServerUIBean != null){
            currentServerUIBean.updateServer(domain);
        }
    }
    /**
     * Try to connect on each servers find in the webconsole.xml and allocate
     * the current domain
     * 
     * @return String
     * @throws NonLocalizedError 
     */
    public String connect() throws NonLocalizedError {
        /*
         * reset ManagementBean
         */
        this.resetManagementBean();
        /*
         * find servers in the configuration file
         */
        Set<ServerTO> servers;
        try {
            final ClassLoader loader = ManagementUIBean.class.getClassLoader();
            URL url = null;
            if(loader != null){
                url = loader.getResource("webconsole.xml");
            } else {
                throw new NonLocalizedError(CLASS_LOADER_ERROR,"class loader not found");
            }
            if(url == null){
                throw new NonLocalizedError(CONFIGURATION_FILE_ERROR,"'webconsole.xml' configuration file not found");
            }
            final File webconsoleFile = new File(url.toURI());
            servers = ConfigHelper.readConfig(webconsoleFile);
        } catch (MalformedURLException e) {
            throw new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage(),e);
        } catch (IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (SAXException e) {
            throw new NonLocalizedError(SAX_ERROR,e.getLocalizedMessage(),e);
        } catch (ParserConfigurationException e) {
            throw new NonLocalizedError(PARSER_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        } catch (FactoryConfigurationError e) {
            throw new NonLocalizedError(FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        } catch (URISyntaxException e) {
            throw new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage(),e);
        }
        final ManagementServiceFactory factory = (ManagementServiceFactory) ManagementServiceFactory
                .getInstance();

        /*
         * For each servers into the configuration file, try to connect it.
         */

        for (ServerTO server : servers) {
            try {
                this.managementService = (ManagementService) factory.getService(server.getHost(),
                        Integer.parseInt(server.getJmxPort()), server.getJmxLogin(), server
                                .getJmxPassword());
            } catch (NumberFormatException e) {
                continue;
            } catch (PetalsServiceException e) {
                continue;
            }
            /*
             * if one server is available recover the domain
             */
            if (this.managementService != null) {
                try {
                    this.domain = TopologyHelper.getDomain(this.managementService.getTopology(),
                            server.getHost(), Integer.parseInt(server.getJmxPort()), server.getJmxLogin(), server.getJmxPassword());                 
                    this.host = server.getHost();
                    this.port = server.getJmxPort();
                    this.login = server.getJmxLogin();
                    this.password = server.getJmxPassword();
                    this.failedConnection = false;
                    return "success";
                } catch (NumberFormatException e) {
                    final NonLocalizedError error = new NonLocalizedError(NUMBER_FORMAT_ERROR,"The port value: "+this.port+" is not properly formatted "+e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (PetalsServiceException e) {
                    // let's try for other declared servers
                    // throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
                } catch (UnknownHostException e) {
                    final NonLocalizedError error = new NonLocalizedError(UNKNOW_HOST_ERROR,"The host: "+this.host+" is unknow or unreachable "+e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                }
            }
       }
        
        /*
         * if no server available to recover topology
         * throw exception
         */

        
         final NonLocalizedError error = new NonLocalizedError(CONFIGURATION_ERROR,"no server available in the configuration file");
         error.setType(IError.FUNCTIONAL_ERROR);
         throw error;
    }
    
    public String serverConnection(final HttpServletRequest request) throws NonLocalizedError{
        final ManagementServiceFactory factory = (ManagementServiceFactory) ManagementServiceFactory
        .getInstance();
        this.failedConnection = true;
        try {
            this.managementService = (ManagementService) factory.getService(this.host,
                    Integer.parseInt(this.port), this.login, this.password);
        } catch (NumberFormatException e) {
            final NonLocalizedError error = new NonLocalizedError(NUMBER_FORMAT_ERROR,"The port value: "+this.port+" is not properly formatted "+e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
        }
        
        /*
         * if one server is available recover the domain
         */
        ServerUIBean serverUIBean = (ServerUIBean) request.getSession().getAttribute("serverUIBean");
        
        if (this.managementService != null) {
            try{
                this.domain = TopologyHelper.getDomain(this.managementService.getTopology(),
                this.host, Integer.parseInt(this.port), this.login, this.password);
            } catch (PetalsServiceException e) {            
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
            } catch (NumberFormatException e) {
                final NonLocalizedError error = new NonLocalizedError(NUMBER_FORMAT_ERROR,"The port value: "+this.port+" is not properly formatted "+e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            } catch (UnknownHostException e) {
                final NonLocalizedError error = new NonLocalizedError(UNKNOW_HOST_ERROR,"The host: "+this.host+" is unknow or unreachable "+e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            }
        }
        
        if(serverUIBean != null){
                this.currentServerName = serverUIBean.getServerName();
        } else {
            if(!this.domain.getServers().isEmpty()){
                this.currentServerName = this.domain.getServers().get(0).getName();
            }
        }
        this.failedConnection = false;
        
        /*
         * Save Server if it's requested
         */
        ServerFeatureUIBean sfUIBean = (ServerFeatureUIBean) request.getSession().getAttribute("serverFeatureUIBean");
        if(sfUIBean != null){
            if(sfUIBean.isWriteServer()){
                    sfUIBean.saveServer(this.host,this.port,this.login,this.password);               
            }
        }
        
        return "success";
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getPort() {
        return port;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public DomainTO getDomain() {
        return this.domain;
    }

    /**
     * Allow to reset the current management bean
     */
    private void resetManagementBean() {
        this.domain = null;
        this.managementService = null;
        this.host = null;
        this.port = null;
        this.login = null;
        this.password = null;
        this.failedConnection = true;
    }

    public void updateDomain() throws NonLocalizedError{
        try {
            this.domain = TopologyHelper.getDomain(this.managementService.getTopology(), this.host,
                    Integer.parseInt(this.port), this.login, this.password);
            this.failedConnection = false;
        } catch (final UnknownHostException e) {
            final NonLocalizedError error = new NonLocalizedError(UNKNOW_HOST_ERROR,"The host: "+this.host+" is unknow or unreachable "+e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final NumberFormatException e) {
            final NonLocalizedError error = new NonLocalizedError(NUMBER_FORMAT_ERROR,"The port value: "+this.port+" is not properly formatted "+e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
        }
    }

    public boolean isFailedConnection() {
        return failedConnection;
    }

    public void setFailedConnection(boolean successConnection) {
        this.failedConnection = successConnection;
    }
    
    public String getCurrentServerName() {
        return currentServerName;
    }

    public void setCurrentServerName(String currentServerName) {
        this.currentServerName = currentServerName;
    }
    
    public String editRuntimeAttributes(final HttpServletRequest request) throws NonLocalizedError{
        this.componentName = request.getParameter("componentName");
        Map<MBeanAttributeInfo, Object> attrs;
        try {
            attrs = this.businessBean.recoverRuntimeAttributes(componentName, this.port, this.password, this.host, this.login);
        } catch (final PetalsServiceException e) {
            if(e.getCause().getClass().equals(RuntimeConfigurationDoesNotExistException.class)){
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,"The component isn't started or runtime configuration for this component doesn't exist");
            } else{
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
            }
        }
        request.getSession().setAttribute("attributes", attrs);
        return "success";
    }
    
    public String validateAttributes(final HttpServletRequest request) throws NonLocalizedError{
        final Map<MBeanAttributeInfo, Object> attributes =
            (Map<MBeanAttributeInfo, Object>) request.getSession()
            .getAttribute("attributes");
        final List<String> occuredErrors = new ArrayList<String>();
        for(final MBeanAttributeInfo info : attributes.keySet()){
            final String s= request.getParameter(info.getName());
            
            Object result = null;
            try{
                if(info.getType().equals("int")||
                   info.getType().equals("Int")||
                   info.getType().equals("integer")||
                   info.getType().equals("Integer")){
                    result = Integer.valueOf(s);
                }
                if((info.getType().equals("string"))||
                   (info.getType().equals("String"))||
                   (info.getType().equals("java.lang.String"))){
                    result = s;
                }
                if(info.getType().equals("float")||
                   info.getType().equals("Float")){
                    result = Float.valueOf(s);
                }
                if(info.getType().equals("double")||
                   info.getType().equals("Double")){
                    result = Double.valueOf(s);
                }
                if(info.getType().equals("short")||
                   info.getType().equals("Short")){
                    result = Short.valueOf(s);
                }
                if(info.getType().equals("bool")||
                   info.getType().equals("Bool")||
                   info.getType().equals("boolean")||
                   info.getType().equals("Boolean")){
                    result = Boolean.valueOf(s);
                }
                if(info.getType().equals("char")||
                   info.getType().equals("Char")||
                   info.getType().equals("character")||
                   info.getType().equals("Character")){
                    result = s.toCharArray();
                }
                if(info.getType().equals("byte")||
                   info.getType().equals("Byte")){
                    result = Byte.valueOf(s);
                }
                if(info.getType().equals("long")||
                   info.getType().equals("Long")){
                    result = Long.valueOf(s);
                }
                attributes.put(info, result);
            }catch(final NumberFormatException e){
                occuredErrors.add("The value '"+s+"' isn't appropriate for '"+
                                 info.getName()+"' attribute, must be '"+
                                 info.getType()+"' type");
            }
        }
        if(occuredErrors.isEmpty()){
            try {
                businessBean.setRuntimeAttributes(this.componentName,this.port,this.password,this.host,this.login,attributes);
            } catch (PetalsServiceException e) {
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
            }
            ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
            managementUIBean.reconnect(request);
            return "success";
        }else {
            request.getSession().setAttribute("occuredErrors", occuredErrors);
            return "error";
        }
    }

    public List<ServerTO> getCheckedServers(){
        final List<ServerTO> serversTO = this.domain.getServers();
        for(final ServerTO serverTO : serversTO){
                serverTO.setAvailable(false);
                ManagementService service = null;
                try {
                    service = (ManagementService) ManagementServiceFactory.getInstance().getService(serverTO.getHost(), Integer.valueOf(serverTO.getJmxPort()),
                            serverTO.getJmxLogin(), serverTO.getJmxPassword());      
                } catch (final PetalsServiceException e) {
                    // continu
                }
                if(service != null){
                    serverTO.setAvailable(true);
                }
       }
        return serversTO;
    }
}
