/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.util;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.opensuit.xmlmap.XmlInstantiator;
import org.ow2.opensuit.xmlmap.XmlSerializer;
import org.ow2.opensuit.xmlmap.schema.SchemasManager;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.xmlbeans.Server;
import org.ow2.petals.tools.webconsole.xmlbeans.Servers;
import org.xml.sax.SAXException;

/**
 * @author aruffie
 *
 */
public class ConfigHelper {

    /**
     * Allow to recover all servers into
     * the webconsole.xml
     * @return Set<ServerTO>
     * @throws MalformedURLException
     * @throws IOException
     * @throws SAXException
     * @throws ParserConfigurationException
     * @throws FactoryConfigurationError
     */
    public static Set<ServerTO> readConfig(File webconsoleFile) throws MalformedURLException, IOException, SAXException, ParserConfigurationException, FactoryConfigurationError{
        final XmlInstantiator xi = XmlInstantiator.instantiate(webconsoleFile.toURI().toURL(), null);
        final Servers servers = (Servers) xi.getRoot();
        final Set<ServerTO> serversTO = new HashSet<ServerTO>();
        if(servers.getServers() != null){
            for(final Server server : servers.getServers()){
                final ServerTO serverTO = new ServerTO();
                serverTO.setHost(server.getHost());
                serverTO.setJmxPort(server.getJmxPort());
                serverTO.setJmxLogin(server.getJmxLogin());
                serverTO.setJmxPassword(server.getJmxPassword());
                serversTO.add(serverTO);
            }
        }
        return serversTO;
    }
    
    /**
     * return a server (by providing the webconsole
     * configuration file), where a channelclient
     * component is installed
     * @param webconsoleFile
     * @return ServerTO
     * @throws MalformedURLException
     * @throws IOException
     * @throws SAXException
     * @throws ParserConfigurationException
     * @throws FactoryConfigurationError
     */
    public static ServerTO findRMIServer(File webconsoleFile) throws MalformedURLException, IOException, SAXException, ParserConfigurationException, FactoryConfigurationError{
        final XmlInstantiator xi = XmlInstantiator.instantiate(webconsoleFile.toURI().toURL(), null);
        final Servers servers = (Servers) xi.getRoot();
            if(servers.getServers() != null){
            for(final Server server : servers.getServers()){
                if(server.isInstalledRmiComponent()){
                    final ServerTO serverTO = new ServerTO();
                    serverTO.setHost(server.getHost());
                    serverTO.setJmxPort(server.getJmxPort());
                    serverTO.setRmiPort(server.getRmiPort());
                    serverTO.setRmiContext(server.getRmiContext());
                    serverTO.setJmxLogin(server.getJmxLogin());
                    serverTO.setJmxPassword(server.getJmxPassword());
                    serverTO.setInstalledRmiComponent(true);
                    return serverTO;
                }
            }
        }
        return null;
    }
    
    /**
     * Return all servers found in the configuration file,
     * into a list type.
     * @param webconsoleFile
     * @return List<Server>
     * @throws MalformedURLException
     * @throws IOException
     * @throws SAXException
     * @throws ParserConfigurationException
     * @throws FactoryConfigurationError
     */
    public static List<Server> readConfiguration(final File webconsoleFile) throws MalformedURLException, IOException, SAXException, ParserConfigurationException, FactoryConfigurationError{
        final XmlInstantiator xi = XmlInstantiator.instantiate(webconsoleFile.toURI().toURL(), null);
        final Servers servers = (Servers) xi.getRoot();
        final List<Server> serverList = new ArrayList<Server>();
        if(servers.getServers() != null){
            for(final Server server : servers.getServers()){
                serverList.add(server);
            }
        }
        return serverList;
    }
    
    public static void writeConfiguration(final File webconsoleFile, final File schema, final List<Server> serverList) throws ParserConfigurationException, IOException{
        
        final Servers servers = new Servers();
        servers.setServers(serverList.toArray(new Server[serverList.size()]));
        final SchemasManager schemasManager = new SchemasManager();
        schemasManager.declareSchema("servers", schema.getPath(), "org.ow2.petals.tools.webconsole.xmlbeans");
        final XmlSerializer xs = XmlSerializer.serialize(schemasManager, servers);
        GeneralHelper.writeXmlFile(xs.getDocument(),webconsoleFile);
    }
}
