/**
 * 
 */
package org.ow2.petals.tools.webconsole.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author aruffie
 * Class allow you to set rights on several
 */
public class FilePermissionHelper {

    /**
     * Allow you to set many several rights on the
     * file provided
     * 1 execute, 2 = write, 3 = write,execute
     * 4 = read, 5 = read/execute
     * 6 = read/write, 7 =read/write/execute
     * asynch : true (Asynchronous mode) /
     * false (Synchronous mode)
     * @param file
     * @param rights
     * @param asynch
     * @throws IOException
     */
    public static void setRights(File file, int[] rights, boolean asynch) throws IOException{
         
      // 4 = read, 2 = write, 1 = execute
      if(file == null){
          throw new IOException("File cannot be null");
      }
      if(rights.length != 3){
          throw new IllegalArgumentException("rights[] must be 3 example: [7][7][7]");
      }
      for(int i : rights){
          if ((i<0)&&(i>7)){
              throw new IllegalArgumentException("Rights must be -1<r<8 ");
          }
      }
      
      Runtime runtime = Runtime.getRuntime();
      
      String command = null;
      if(System.getProperty("os.name").equals("Linux")){
      //pour unix
        command= "chmod -R "+rights[0]+rights[1]+rights[2]+" "+file.getAbsolutePath();
      
      }
      if(System.getProperty("os.name").equals("Windows")){
          //pour windows
          //throw new IllegalStateException("Plateform: "+System.getProperty("os.name")+" not supported");
          //runtime.exec("cacls.exe "+file.getAbsolutePath()+" "+permission});
      }
      
      /*
       * if the runtime is available (is know) and
       * if the command isn't null process the
       * command execution
       */
      if((command != null) && (runtime != null)){
          Process process = runtime.exec(command);
          
          if(!asynch){
              synchronized(process){
                  try {       
                    process.waitFor();
                  } catch (InterruptedException e) {
                    e.printStackTrace();
                  }
              }
          }
      }
   }
    
    
   /**
    * Function allow you to set rights
    * read/write/exec, on the zip provided
    * 1 execute, 2 = write, 3 = write,execute
    * 4 = read, 5 = read/execute
    * 6 = read/write, 7 =read/write/execute
    * asynch : true (Asynchronous mode) /
    * false (Synchronous mode)
    * @param zip
    * @param rights
    * @param asynch
    * @throws IOException
    */
   public static void setRightInZip(File zip, int[] rights, boolean asynch) throws IOException{
       
       Runtime runtime = Runtime.getRuntime();
       
       String command = null;
       String dirPath = zip.getAbsolutePath().substring(0, (int)zip.getAbsolutePath().length()-4);
       
       if(System.getProperty("os.name").equals("Linux")){
       //pour unix      
         command= "unzip -X "+zip.getAbsolutePath()+" -d "+dirPath;
       }
       if(System.getProperty("os.name").equals("Windows")){
           
       }
       //unzip all file in /zip.name directory
       
       Process process = null;
       /*
        * if the runtime is available (is know) and
        * if the command isn't null process the
        * command execution
        */
       if((command != null) && (runtime != null)){
           process = runtime.exec(command);
           
           if(!asynch){
               synchronized(process){
                   try {       
                     process.waitFor();
                   } catch (InterruptedException e) {
                     // TODO Auto-generated catch block
                     e.printStackTrace();
                   }
               }
           }
       }
       
       
       
       if(System.getProperty("os.name").equals("Linux")){
           //pour unix      
             command= "zip -r "+zip.getAbsolutePath();
      }
       if(System.getProperty("os.name").equals("Windows")){
           
       }
       File directory = new File(dirPath);
       //Re-allow all rights
       final List<String> files = new ArrayList<String>();
       for(final File f: directory.listFiles()){
           if((f.isFile())||(f.getName().equals("META-INF"))){
               setRights(f, rights, false);   
               command+=" "+/*+dirPath+"/"+*/f.getCanonicalPath();               
           } else {
               setRights(f, rights, false);   
               command+=" "+/*+dirPath+"/"+*/f.getCanonicalPath();
           }
           if(!f.isDirectory()){
               files.add(f.getName());
           }
       }
      final String zipAbsolutePath = zip.getAbsolutePath();
      
      zip.delete();
      for(final File f: directory.listFiles()){
          if(f.getName().equals("home")){
              f.delete();
              
          }
      }
      
      final String zipPath = zip.getAbsolutePath().substring(0,zip.getAbsolutePath().indexOf(".zip"));
      command = "zip -r "+zip.getAbsolutePath()+" "+zipPath;
      //rezip all files
      
      /*
       * if the runtime is available (is know) and
       * if the command isn't null process the
       * command execution
       */
      if((command != null) && (runtime != null)){
          process = runtime.exec(command);
          
          if(!asynch){
              synchronized(process){
                  try {       
                    process.waitFor();
                  } catch (InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                  }
              }
          }
      }
      
      creationZip(zipAbsolutePath ,files.toArray(new String[files.size()]) );
      //Eligeable for gc
      command = null;
      dirPath = null;
      directory = null;
       
   }
   
   private static void creationZip(final String zipPath, final String[] cheminsFichiers)
   {
       int BUFFER = 2048;
       try {
   BufferedInputStream origin = null;
   FileOutputStream dest = new FileOutputStream(zipPath);
   ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(dest));
   byte data[] = new byte[BUFFER];
   for (int i=0; i<cheminsFichiers.length; i++) {
   System.out.println("Adding: "+cheminsFichiers[i]);
   FileInputStream fi = new FileInputStream(cheminsFichiers[i]);
   origin = new BufferedInputStream(fi, BUFFER);
   ZipEntry entry = new ZipEntry(cheminsFichiers[i]);
   out.putNextEntry(entry);
   int count;
   while((count = origin.read(data, 0, 
   BUFFER)) != -1) {
   out.write(data, 0, count);
   }
   origin.close();
   }
   out.close();
   } catch(Exception e) {
   e.printStackTrace();
   }
   }  
}

